<template>
  <div class="bbsl-common-page">
    <div class="title-box">
      <public-title title="路障设备示例图" />
    </div>

    <div class="reference-image-box">
      <div v-loading="loading" class="reference-image-component" element-loading-background="rgba(0, 0, 0, 0.8)"
           element-loading-spinner="el-icon-loading" element-loading-text="设备请求中..">
        <!--        <div class="demo" :style="{'&#45;&#45;font-size':'40px'}">111</div>-->
        <div class="legend-box">
          <div class="legend-box-title">路障设备状态图例</div>
          <div class="legend-box-content">
            <div class="row">
              <i style="background-color: #4fc200" />
              <span style="color: #4fc200">升起状态</span>
            </div>
            <div class="row">
              <i style="background-color: #E6A23C" />
              <span style="color: #E6A23C">降落状态</span>
            </div>
            <div class="row">
              <i style="background-color: #cccccc" />
              <span style="color: #cccccc">离线状态</span>
            </div>
          </div>
          0
        </div>

        <div class="equipment-box">
          <div v-for="(item, index) in equipmentData" :key="index" :ref="`equipment${item.id}`"
               @click="equipmentClick(item)" @mouseenter="equipmentMouseenter(item)"
               @mouseleave="equipmentMouseleave(item)">
            <div class="equipment-group">
              <i v-for="_index in 5" :key="_index" :class="[
                'equipment',
                item.showTooltip || item.showPopover
                  ? 'equipment-hover-active'
                                        : '',
                                    dataInitialization ? item.deviceClass : '',
                                ]" />
              <div class="device-info">
                <div class="equipment-name">
                  {{ item.name }}
                </div>
                <div class="equipment-info">
                  <div class="equipment-status">
                    设备状态：{{
                      dictionaryEscape(
                        statusOpt,
                        equipmentInfo.bollardStatus
                      ) || '-'
                    }}
                  </div>
                  <div class="equipment-code">
                    设备标识：{{ item.code }}
                  </div>
                </div>
                <div class="equipment-cation">
                  <el-button v-if="item.bollardStatus === '99'" size="mini" type="success" @click="
                    showDialog('confirmDialog', item)
                                        ">
                    上升
                  </el-button>
                  <el-button
                    v-else-if="item.bollardStatus === '4'"
                    v-show="showButton"
                    disabled
                    size="mini"
                    type="info"
                  >

                  </el-button>
                  <el-button v-else size="mini" type="warning" @click="
                    showDialog('confirmDialog', item)
                                        ">下降
                  </el-button>
                </div>
              </div>
              <div class="rounded-triangle"></div>
            </div>
          </div>
        </div>
      </div>
      <div class="reference-video-box">
        <DHVideoPlayPlugin ref="DHVideoIframe" :autoPlay="true" :splitNum="1" class="video-container"
                           @ready="playVideo" />
        <!-- <div class="video-box" v-for="item in videoArr" :key="item.id">
          <video-component ref="videoComponent" :videoInfo="item" />
        </div> -->
        <!-- <div class="video-box">
          <video-component ref="videoComponent1" />
        </div> -->
      </div>
    </div>

    <el-dialog :visible.sync="confirmDialog" append-to-body title="请输入验证码" top="30vh" width="25%"
               @closed="confirmDialogClosed">
      <div class="dialogBox">
        <el-form ref="confirmForm" :model="confirmForm" :rules="confirmRules" label-suffix=":" label-width="auto">
          <el-form-item label="验证码" prop="authCode">
            <el-input v-model="confirmForm.authCode" placeholder="请输入右侧的验证码" style="width: 100%">
              <template v-slot:append>
                <span class="code-span" title="刷新验证码" @click="randyzm()">
                  {{ confirmForm.code }}
                </span>
              </template>
            </el-input>
          </el-form-item>
        </el-form>
      </div>

      <template v-slot:footer>
        <el-button :loading="confirmLoading" :type="equipmentInfo.bollardStatus === '99'
            ? 'danger'
                            : 'success'
                    " @click="risingAndFalling">
          {{
            equipmentInfo.bollardStatus === '99' ? '上 升' : '下 降'
          }}
        </el-button>
        <el-button @click="confirmDialog = false">取 消</el-button>
      </template>
    </el-dialog>
  </div>
</template>

<script>
import VideoComponent from './components/VideoComponent/videoIndex.vue'
import PublicTitle from './components/PublicTitle/index.vue'
import dictUtils from '@/components/dict/DictSelectUtils'
import riotBarricade from '@/api/device/riotBarricade'
import {dictionaryEscape} from '@/utils/dictionary'
import DHVideoPlayPlugin from '@/views/Device/deviceMonitor/components/DHVideoPlayPlugin.vue'

export default {
  name: 'BarricadeEquipment',
  components: {
    PublicTitle,
    VideoComponent,
    DHVideoPlayPlugin
  },
  data() {
    const validateAuthCode = (_, authCode, callback) => {
      if (
        this.confirmForm.code.toLowerCase() !== authCode.toLowerCase()
      ) {
        callback(new Error('验证码输入错误，请重新输入'))
      } else {
        callback()
      }
    }

    return {
      showButton: false,
      reference: {},
      showPopover: false,
      equipmentInfo: {},
      loading: true, // loading
      dataInitialization: false, // 数据初始化
      equipmentData: [], // 设备数据
      statusOpt: dictUtils.getDictItemsFromCache('sbzt') || [],
      actionOpt: dictUtils.getDictItemsFromCache('dzbs'),
      confirmDialog: false, // 设备控制弹层，
      confirmLoading: false,
      confirmForm: {
        code: '',
        authCode: ''
      },
      confirmRules: {
        authCode: [
          {
            required: true,
            message: '验证码不能为空',
            trigger: 'blur'
          },
          {validator: validateAuthCode, trigger: 'blur'}
        ]
      },
      videoArr: []
    }
  },
  created() {
    this.getEquipmentDataData()
  },
  methods: {
    playVideo() {
      this.$nextTick(() => {
        console.log('准备播放this.$refs', this.$refs)
        // 控件名称，监控点编码
        this.$refs.DHVideoIframe.createPlay(
          'ctrl-sssb-barricade-top',
          '50011201561322060012',
          'top'
        )
        this.$refs.DHVideoIframe.createPlay(
          'ctrl-sssb-barricade-bottom',
          '50011201561322010013',
          'bottom'
        )
      })
    },
    dictionaryEscape,

    // 获取设备数据
    getEquipmentDataData() {
      const queryParam = {pageBean: {pageSize: -1, page: 1}}
      riotBarricade.loadBarricade(queryParam, (res) => {
        // 去重后数据
        const deduplicationData = [
          ...new Map(
            res.rows.flatMap((item) => {
              let {code = '', name = '', ...i} = item
              if (!code || typeof code !== 'string') return []
              code = code.split('-')[0]
              name = name.split('#')[0]
              return [[code, {...i, code, name}]]
            })
          ).values()
        ]

        const midIndex = Math.floor(deduplicationData.length / 2) // 中间索引，适用于奇数和偶数长度的数组
        this.equipmentData = deduplicationData.map(
          (item, index, arr) => {
            let placement
            if (index < midIndex) {
              // 前半部分
              placement = 'top-end'
            } else if (index > midIndex) {
              // 后半部分
              placement = 'top-start'
            } else {
              // 特殊处理中间元素，如果需要
              if (arr.length % 2 !== 0) {
                placement = 'top'
              } else {
                // 如果数组长度为偶数，中间索引元素属于前半部分
                placement = 'top-start'
              }
            }
            return {
              ...item,
              placement,
              deviceClass:
                item.bollardStatus === '98'
                  ? 'equipment-rise'
                  : item.bollardStatus === '4'
                    ? 'equipment-gray'
                    : 'equipment-decline',
              showTooltip: false, // 是否展示Tooltip
              showPopover: false, // 是否展示Popover
              isState: !!item?.isLock
            }
          }
        )
        this.$nextTick((_) => {
          setTimeout((_) => (this.dataInitialization = true), 0)
          setTimeout((_) => {
            this.equipmentData.forEach(
              (item) => (item.showTooltip = true)
            )
          }, 500)
        })
        const {videoCode} = this.equipmentData[0]
        let codeArr = videoCode.split(',')
        codeArr.forEach((code, index) => {
          this.videoArr.push({
            code,
            id: index + 1,
            styleId: `my-video${index + 1}`
          })
        })
        this.loading = false
        // this.$refs.videoComponent?.getUrlAndPlay(this.equipmentData[0] || {}, 0)
        // this.$refs.videoComponent1?.getUrlAndPlay(this.equipmentData[0] || {}, 1)
      })
    },

    // 设备点击
    equipmentClick(item) {
      this.equipmentData.forEach((i) => {
        item.showTooltip = false
        if (i.id === item.id) {
          i.showPopover = !i.showPopover
        } else {
          i.showPopover = false
        }
      })
      // 这个操作是为了避免与源码中的点击reference doToggle方法冲突
      if (this.equipmentInfo === item && this.showPopover) return
      this.showPopover = false
      this.equipmentInfo = item
      // reference是需要获取dom的引用
      this.reference = this.$refs[`equipment${item.id}`][0]
    },

    // 设备鼠标移入
    equipmentMouseenter(item) {
      if (item.showPopover) return
      item.showTooltip = true
    },

    // 设备鼠标移出
    equipmentMouseleave(item) {
      item.showTooltip = false
    },

    // popover隐藏
    popoverHide() {
      this.equipmentInfo.showPopover = false
    },

    // 展示弹层
    showDialog(dialogName, item) {
      if (item.bollardStatus === '4') {
        this.$message.warning('当前设备为置灰状态，不可操作')
        return
      }
      if (dialogName === 'confirmDialog' && item) {
        this.equipmentInfo = item
      }
      this.randyzm((_) => (this[dialogName] = true))
    },

    // 关闭弹层后
    confirmDialogClosed() {
      this.confirmForm = this.$options.data().confirmForm
      this.$nextTick((_) => this.$refs.confirmForm.clearValidate())
    },

    // 验证码方法
    randyzm(callback) {
      //生成随机验证码
      let code = ''
      let coden = ''
      let codeLength = 4 //验证码的长度
      let random = [
        0,
        1,
        2,
        3,
        4,
        5,
        6,
        7,
        8,
        9,
        'A',
        'B',
        'C',
        'D',
        'E',
        'F',
        'G',
        'H',
        'I',
        'J',
        'K',
        'L',
        'M',
        'N',
        'O',
        'P',
        'Q',
        'R',
        'S',
        'T',
        'U',
        'V',
        'W',
        'X',
        'Y',
        'Z',
        'a',
        'b',
        'c',
        'd',
        'e',
        'f',
        'g',
        'h',
        'i',
        'j',
        'k',
        'l',
        'm',
        'n',
        'o',
        'p',
        'q',
        'r',
        's',
        't',
        'u',
        'v',
        'w',
        'x',
        'y',
        'z'
      ] //随机数
      for (let i = 0; i < codeLength; i++) {
        //循环操作
        let index = Math.floor(Math.random() * 62) //取得随机数的索引（0~61）
        code += random[index] //根据索引取得随机数加到code上
        coden += random[index] //根据索引取得随机数加到code上
      }
      this.confirmForm.code = coden //把code值赋给验证码
      callback && callback()
    },

    // 上升 || 下降
    risingAndFalling() {
      this.$refs.confirmForm?.validate((valid, object) => {
        if (!valid) {
          this.$message.warning(
            `有${
              Object.keys(object).length
            }个字段未通过校验，请正确填写表单内容。`
          )
        } else {
          this.confirmLoading = true
          let data = {
            action:
              this.actionOpt.find((i) => {
                return (
                  i.name ===
                  (this.equipmentInfo.bollardStatus === '99'
                    ? '上升'
                    : '下降')
                )
              })?.value || '',
            controller_id: this.equipmentInfo.code,
            deviceCode: this.equipmentInfo.code,
            deviceName: this.equipmentInfo.name,
            operator_name:
              JSON.parse(
                window.sessionStorage.getItem('currentUser')
              )?.username || ''
          }
          riotBarricade.operationBarricade(data, (_) => {
            this.confirmDialog = false
            this.getEquipmentDataData()
            setTimeout((_) => (this.confirmLoading = false), 0)
          })
        }
      })
    }
  }
}
</script>

<style lang="scss" scoped>
@import 'element-variables.scss';

.bbsl-common-page {
  display: flex;
  flex-direction: column;
  background-color: #fff;
  overflow-y: auto;

  &::-webkit-scrollbar {
    display: none;
  }

  //.video-box {
  //  min-height: 700px;
  //  background-color: #fff;
  //  box-shadow: 0 0 12px 0 rgba($color: #000000, $alpha: 0.1);
  //  display: flex;
  //  flex-direction: column;
  //
  //  .video-box-content {
  //    padding: 15px 20px;
  //    flex: 1;
  //  }
  //}

  .title-box {
    top: 0;
    position: sticky;
    z-index: 10;
    background-color: #fff;
  }

  .reference-image-box {
    display: flex;
    column-gap: 10px;
    padding: 15px 20px;
    flex: 1;
    height: 0;
    min-height: 700px;
    border-radius: 3px;

    .reference-image-component {
      width: 65%;
      height: 100%;
      background: url("~@/assets/img/example-diagram-of-lifting-column(1).png") no-repeat 0 0;
      background-size: 100% 100%;
      position: relative;
      border-radius: 2px;

      .legend-box {
        left: 2%;
        top: 3%;
        position: absolute;
        display: flex;
        flex-direction: column;
        row-gap: 15px;
        padding: 15px 25px 60px 15px;
        background-color: rgba(52, 52, 52, 0.8);
        border-radius: 5px;

        .legend-box-title {
          font-size: 17px;
          color: #fff;
          font-weight: bold;
        }

        .legend-box-content {
          display: flex;
          flex-direction: column;
          row-gap: 10px;

          .row {
            display: flex;
            align-items: center;
            column-gap: 12px;

            i {
              display: inline-block;
              width: 16px;
              height: 16px;
              border-radius: 3px;
            }

            span {
              font-size: 16px;
              margin-bottom: 1px;
            }
          }
        }
      }

      .video-box {
        width: 35%;
        min-width: 400px;
        height: 35%;
        min-height: 220px;
        // position: absolute;
        // top: 0;
        // right: 0;
      }

      .equipment-box {
        display: flex;
        column-gap: 35px;
        position: absolute;
        left: 24%;
        bottom: 23.5%;
        //transform: rotate(-3deg);
        cursor: pointer;

        > div {
          display: flex;
          flex-direction: column;
          justify-content: flex-end;

          // &:nth-child(2) {
          //   transform: translateY(-2px);
          // }

          // &:nth-child(3) {
          //   transform: translateY(-4px);
          // }

          // &:nth-child(4) {
          //   transform: translateY(-6px);
          // }
          &:nth-child(1) {
            .equipment-group {
              .device-info {
                bottom: calc(100% + 8px);
                right: 0;
              }
            }
          }

          &:nth-child(2) {
            .equipment-group {
              .device-info {
                top: calc(100% + 8px);
                right: 0;
              }

            }
          }

          &:nth-child(3) {
            .equipment-group {
              .device-info {
                bottom: calc(100% + 8px);
                left: 0;
              }

            }
          }

          &:nth-child(4) {
            .equipment-group {
              .device-info {
                top: calc(100% + 8px);
                left: 0;
              }
            }
          }

          &:nth-child(2n) {
            .equipment-group {
              .rounded-triangle {
                left: 52px;
                top: calc(100% - 8px);
                border-top: 8px solid transparent;
                border-bottom: 8px solid rgba(135, 138, 154, .7);
                border-left: 8px solid transparent;
                border-right: 8px solid transparent;
              }
            }
          }

          &:nth-child(2n+1) {
            .equipment-group {
              .rounded-triangle {
                left: 52px;
                bottom: calc(100% - 8px);
                border-top: 8px solid rgba(135, 138, 154, .7);
                border-bottom: 8px solid transparent;
                border-left: 8px solid transparent;
                border-right: 8px solid transparent;
              }
            }
          }
        }

        .equipment-group {
          display: flex;
          position: relative;
          column-gap: 10px;

          .equipment {
            width: 16px;
            height: 0;
            font-size: var(--font-size, 20px);
            opacity: 0.7;
            transition: opacity 0.15s, height 0.5s;

            &.equipment-hover-active {
              opacity: 1;
            }

            // 上升样式
            &.equipment-rise {
              height: 45px;
              background-color: #ff6200;
            }

            // 下降样式
            &.equipment-decline {
              height: 20px;
              background-color: #4fc200;
            }

            // 置灰样式
            &.equipment-gray {
              height: 20px;
              background-color: #cccccc;
            }
          }

          .device-info {
            cursor: default;
            position: absolute;
            width: 230px;
            min-height: 100px;
            padding: 6px;
            background: rgba(135, 138, 154, .7);
            border-radius: 3px;

            .equipment-name {
              position: relative;
              height: 24px;
              padding-left: 8px;
              line-height: 24px;
              font-size: 15PX;
              color: #fff;

              &::after {
                position: absolute;
                left: 0;
                top: 4px;
                content: '';
                width: 3px;
                height: 16px;
                background: #4e74e0;
              }
            }

            .equipment-info {
              > div {
                width: 100%;
                height: 24px;
                line-height: 24px;
                color: #fff;
              }
            }

            .equipment-cation {
              display: flex;
              margin-top: 10px;
              justify-content: end;
            }
          }

          .rounded-triangle {
            position: absolute;
          }
        }
      }
    }

    .reference-video-box {
      width: 35%;
      height: 100%;

      .video-box {
        width: 100%;
        min-width: 400px;
        height: calc(50% - 5px);
        min-height: 220px;

        // position: absolute;
        // top: 0;
        // right: 0;
        &:last-child {
          margin-top: 10px;
        }
      }

    }

  }
}

.popover-box {
  .el-popover__title {
    display: flex;
    align-items: center;
    column-gap: 10px;

    &::before {
      content: '';
      width: 3px;
      height: 16px;
      background-color: #4e74e0;
    }
  }

  .popover-content {
    margin-bottom: 15px;
    display: flex;
    flex-direction: column;
    row-gap: 5px;

    .row {
      display: flex;

      .label {
        font-size: 14px;
      }

      .value {
        flex: 1;
        font-size: 14px;
      }
    }
  }

  .popover-bottom {
    display: flex;
    justify-content: flex-end;
  }

}

::v-deep .el-input-group__append {
  padding: 0 !important;

  .code-span {
    display: inline-block;
    text-align: center;
    width: 90px;
    font-size: 20px;
    height: 100%;
    line-height: 32px;
    box-sizing: border-box;
    color: #000000;
    background-color: #efefef;
    cursor: pointer;
  }
}

.video-container {
  height: 100%;
  width: 100%;
}
</style>

<style lang="scss">
.popover-box {
  .el-popover__title {
    display: flex;
    align-items: center;
    column-gap: 6px;

    &::before {
      content: '';
      width: 4px;
      height: 18px;
      background-color: #4e74e0;
    }
  }
}
</style>
